﻿// Copyright (c) Pixel Crushers. All rights reserved.

using UnityEngine;
using System;
using DG.Tweening;
using BlueGoo.Dialogue;
using BlueGoo.Utils;

namespace PixelCrushers.DialogueSystem {
	/// <summary>
	/// Standard UI implementation of IBarkUI.
	/// </summary>

	//[AddComponentMenu("")] // Use wrapper.
	public class CustomStandardBarkUI : StandardBarkUI {
		// Private vars
		private RectTransform rect;

		[Header("Custom settings ")]

		[SerializeField]
		private Ease curveShow;

		[SerializeField]
		private Ease curveHide;
		
		private CustomUnityUITypewriterEffect typewriter;
		

		private enum BubbleType {
			Speak,
			Think
		}

		[SerializeField]
		private BubbleType bubbleType = BubbleType.Speak;


		private Sequence mySequence;
		private Subtitle subtitle;
		
		protected override void Start() {
			rect = canvas.GetComponent<RectTransform>();
			typewriter = GetComponentInChildren<CustomUnityUITypewriterEffect>();
			// Hide on start
			canvasGroup.alpha = 0;
			rect.localScale = new Vector3(0,0,0);
		}

		public override void Bark(Subtitle setSubtitle) {
			subtitle = setSubtitle; // Set temp variable, as Invoke to SetText can't take any parameters
			
			if (ShouldShowText(subtitle)) {

				Debug.Log(Time.frameCount + ": " + name + ": BARKUI = SHOW!" , gameObject);

				//Debug.Log(Time.time.ToString("F1")  + ": Show bark");
				//SetUIElementsActive(true); // Detta aktiverar objektet, om Play on enable är på, så börjar den automatiskt
				//Invoke(nameof(EnableTypeWriterAfterDelay), durationShow); // Vänta lite innan den börjar skriva

				// Cancel if it's trying to hide it already
				CancelInvoke(nameof(Hide));
				
				// TODO: Vad är det som gömmer den? Den ska visas 2.85s, men visas i 3.2s, så något overridar..

				//Debug.Log("Start Tween - before: " + MasterManager.SettingsManager.durationShow + " alpha = " + canvasGroup.alpha);
				
				mySequence.Kill();
				mySequence = DOTween.Sequence().OnComplete(UIShowComplete);
				var startTime = 0f; 
				
				// If visible from another bark, first hide quickly and show again
				if (canvasGroup.alpha > 0) {
					mySequence.Insert(0,rect.DOScale(new Vector3(0, 0, 0), MasterManager.ConfigManager.durationHide).SetEase(curveHide));
					mySequence.Insert(0,DOTween.To(() => canvasGroup.alpha, x => canvasGroup.alpha = x, 0f, MasterManager.ConfigManager.durationHide).SetEase(curveHide));
					startTime = MasterManager.ConfigManager.durationHide; // delay start to let it hide first
				}
				mySequence.Insert(startTime,rect.DOScale(new Vector3(1, 1, 1), MasterManager.ConfigManager.durationShow).SetEase(curveShow));
				mySequence.Insert(startTime,DOTween.To(() => canvasGroup.alpha, x => canvasGroup.alpha = x, 1.0f, MasterManager.ConfigManager.durationShow).SetEase(curveShow));

				// Set text after it has shut down (or direct if it wasn't visible) - to allow it to change height of bubble
				Invoke(nameof(SetText),startTime);
				
				// Start typewriter slightly before it's shown
				Invoke(nameof(StartTypewriter),startTime + MasterManager.ConfigManager.durationShow / 2);
				
				
				var barkDuration = Mathf.Approximately(0, duration) ? DialogueUtils.GetSubtitleDuration(subtitle.formattedText.text) : duration;
				//Debug.Log(Time.time.ToString("F1")  + ": Show bark for " + barkDuration + " seconds");

				Debug.Log("waitUntilSequenceEnds = " + (waitUntilSequenceEnds ? "true" : "false"));
				Debug.Log("waitForContinueButton = " + (waitForContinueButton ? "true" : "false"));
				if (!(waitUntilSequenceEnds || waitForContinueButton)) {
					// Denna körs inte då den väntar på sequence end
					Debug.Log(Time.time.ToString("F1")  + ": Invoke Hide auro in " + (barkDuration + MasterManager.ConfigManager.durationAfterBark) + " seconds");
					Invoke(nameof(Hide), barkDuration + MasterManager.ConfigManager.durationAfterBark);
				}
				
				if (waitUntilSequenceEnds) numSequencesActive++;
				doneTime = waitForContinueButton ? Mathf.Infinity : (DialogueTime.time + barkDuration + MasterManager.ConfigManager.durationAfterBark);
			}
		}

		private void SetText() {
			SetUIElementsActive(false);

			string subtitleText = subtitle.formattedText.text;

			if (includeName) {
				if (nameText != null) {
					nameText.text = subtitle.speakerInfo.Name;
				}
			} else {
				if (nameText != null && nameText.gameObject != null) nameText.gameObject.SetActive(false);
			}

			if (showPortraitImage && subtitle.speakerInfo.portrait != null) {
				Tools.SetGameObjectActive(portraitImage, true);
				portraitImage.sprite = subtitle.speakerInfo.portrait;
			} else {
				Tools.SetGameObjectActive(portraitImage, false);
			}

			if (barkText != null) {
				barkText.text = subtitleText;
				barkText.color = new Color(0,0,0,0);
				//Debug.Log("Set text to " + subtitleText);
			}
			SetUIElementsActive(true);
		}

		private void StartTypewriter() {
			barkText.color = new Color(0,0,0,1);
			typewriter.Play();
		}
		
		private void UIShowComplete() {
			Debug.Log("UIShowComplete: " + name + " : " + MasterManager.ConfigManager.durationShow + " alpha = " + canvasGroup.alpha, gameObject);
			// Start typewriter
		}

		private void UIHideComplete() {
			Debug.Log("UIHideComplete: " + name + " : " + MasterManager.ConfigManager.durationShow + " alpha = " + canvasGroup.alpha, gameObject);
			canvas.enabled = false;
		}
		/*
		private void EnableTypeWriterAfterDelay() {
			SetUIElementsActive(true);
		}
		*/


		public override void Hide() {
			Debug.Log(Time.time.ToString("F1") + name + ": BARKUI = Hide running!" , gameObject);

			mySequence.Kill();
			mySequence = DOTween.Sequence().OnComplete(UIHideComplete);
			mySequence.Insert(0,rect.DOScale(new Vector3(0, 0, 0), MasterManager.ConfigManager.durationHide).SetEase(curveHide));
			mySequence.Insert(0,DOTween.To(() => canvasGroup.alpha, x => canvasGroup.alpha = x, 0f, MasterManager.ConfigManager.durationHide).SetEase(curveHide));

			numSequencesActive = 0;
			doneTime = 0;

			Debug.Log("<color=green>CONTINUE</color>");
		}
	}
}